//
//  ClScrCategories.m
//  TaxControl
//
//  Created by Javier Cava García on 12/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrCategories.h"
#import "ClScrNovaCategoria.h"
#import "ClComunes.h"



@interface ClScrCategories ()

@end


@implementation ClScrCategories
{
    ClComunes *funcComunes;
}


// Array de Categories
@synthesize CategoriesArray;
// Core data
@synthesize managedObjectContext;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    @try {
        // Establim les categories
        NSMutableArray *mutableFetchResults  = [funcComunes setCategories:managedObjectContext];
        [self setCategoriesArray:mutableFetchResults];
        // Refresh del tableview
        [_tblCategories reloadData];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblCategories addGestureRecognizer:tapEdit];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        
        CGPoint tapLocation = [cell locationInView:self.tblCategories];
        NSIndexPath *indexPath = [self.tblCategories indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de nova categoria pero en mode edicio
            ClScrNovaCategoria *editCatView = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewCategoriaEdit"];
            // Activem el mode d'edicio
            editCatView.edicio = @1;
            // Obtenim el id de la categoria a editar
            Categoria *lCategoria  = (Categoria *)[CategoriesArray  objectAtIndex: indexPath.row];
            editCatView.CategoriaEdit  = lCategoria;
            //NSManagedObjectID *moID = [lCat objectID];
            editCatView.navigationItem.prompt = @"Edició Categoria";
            // Cridem la pantalla
            [self.navigationController pushViewController:editCatView animated:YES];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [CategoriesArray count];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
  
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    // Obtenim el text de la categoria
    Categoria *lCat  = (Categoria *)[CategoriesArray  objectAtIndex: indexPath.row];
    // Id intern de l'objecte
//    NSURL *objURL       = [[lCat objectID] URIRepresentation];
//    NSString *objId     = [NSString stringWithFormat:@"ID:[ %@ ] ",[objURL lastPathComponent]];
//    NSString *cellLabel = [objId stringByAppendingString:lCat.txt ];
    
    // Establim el text de la Cell
    [cell.textLabel setText:lCat.txt];
    return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSError *error = nil;
    @try {
        
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            // Abans d'eliminar, comprobem integritat referencial
            Categoria *lCat  = (Categoria *)[CategoriesArray  objectAtIndex: indexPath.row];
            NSMutableArray *ref = [funcComunes get_references_categoria:lCat :managedObjectContext];
            if ( [ref  count] > 0)
            {
                [funcComunes msg_pantalla:@"La Categoria encara s'utilitza, no es pot eliminar!"];
            }
            else
            {
                [managedObjectContext deleteObject:lCat];
                [managedObjectContext save:&error];
                // Elimina l'entrada de l'array
                [CategoriesArray removeObjectAtIndex:indexPath.row];
                // Elimina la fila del tableview
                [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
                [_tblCategories reloadData];
            }
        }
        else if (editingStyle == UITableViewCellEditingStyleInsert) {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Liberamos memoria
-(void)viewWillDisappear:(BOOL)animated
{
   
}


@end
