//
//  ClScrExportacio.m
//  TaxControl
//
//  Created by Javier Cava García on 10/11/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrExportacio.h"
#import "ClComunes.h"
#import "Trimestre.h"
#import "Trajecte.h"
#import "ImportFixe.h"
#import "Quilometre.h"
#import "Despesa.h"
#import "Factura.h"
#import "TipusDIVA.h"
#import "Mail.h"



@interface ClScrExportacio ()

{
    NSUInteger no_despeses;
    NSUInteger no_factures;
}
@end

@implementation ClScrExportacio
{
    ClComunes *funcComunes;
    Trimestre *trimestre_sel;
    NSString *docDirectory;
    NSNumber *preuxKm;
}

// Array de Trimestres
@synthesize TrimestresArray;
@synthesize kmArray;
@synthesize MailArray;
// Core data
@synthesize managedObjectContext;

@synthesize txtTrimestre;
@synthesize progres;
@synthesize Timer;
@synthesize lProgress;

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}


-(void)viewWillAppear:(BOOL)animated
{
    self.progres.progress = 0;
    lProgress.text = nil;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Do any additional setup after loading the view.
        // Uncomment the following line to preserve selection between presentations.
        //self.clearsSelectionOnViewWillAppear = NO;
        // set  la imatge de fons
        self.view.backgroundColor = [UIColor colorWithPatternImage:[UIImage imageNamed:@"Background"]];
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        // Carreguem els trimestres
        NSMutableArray *mutableFetchResults = [funcComunes set_trimestres:managedObjectContext];
        [self setTrimestresArray:mutableFetchResults];
        mutableFetchResults = nil;
        // Carreguem el mail
        mutableFetchResults = [funcComunes set_mail:managedObjectContext];
        [self setMailArray:mutableFetchResults];
        mutableFetchResults = nil;
        // Configurem step trimestres
        [self configura_step_trimestres];
        NSUInteger ini = 0;
        [self setvalortrimestre:&ini];
        // Carreguem les dades que hi ha
        mutableFetchResults = [funcComunes set_quilometres:managedObjectContext];
        [self setKmArray:mutableFetchResults];
        if ( kmArray != nil)
        {
            Quilometre *impKm = (Quilometre *) [kmArray objectAtIndex:0];
            preuxKm = impKm.preu;
        }
        else [funcComunes msg_pantalla:@"Imposible exportar Trajectes, no s'ha definit preu x km"];
        self.progres.progress = 0;
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}



// Configura step trimestres
-(void)configura_step_trimestres
{
    [self.stepTrimestres setStepValue: 1];
    [self.stepTrimestres setMinimumValue:0];
    if ( TrimestresArray == nil )
    {
        [self.stepTrimestres setMaximumValue:0];
    } else [self.stepTrimestres setMaximumValue:[TrimestresArray count]-1];
    
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (IBAction)stepTrimestreValChg:(UIStepper *)sender {
    
    NSUInteger stepvalue = [sender value];
    if ( ( stepvalue <= [TrimestresArray count]-1 ))
        [self setvalortrimestre:&stepvalue];
    
}




// Estableix el valor
-(void)setvalortrimestre:(NSUInteger *)index
{
    
    Trimestre *lTrimestre  = (Trimestre *)[TrimestresArray   objectAtIndex:  *index];
    txtTrimestre.text = lTrimestre.txt;
    // Actualitzem el trimestre seleccionat
    trimestre_sel = lTrimestre;
    
}

// Exportació de dades
- (IBAction)btnExportacio:(id)sender {
    
    @try {
        no_despeses = 0;
        no_factures = 0;
        // Progress
        self.progres.progress = 0;
        // Obtenim el directori
        NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
        docDirectory = [paths objectAtIndex:0];
        // Generem el fitxer CSV
        // CSV Despeses
        lProgress.text = @"Exportant despeses i trajectes ...";
        NSString *fileDespeses = [self crea_csv_despeses];
        self.progres.progress = 0.5;
        // CSV Factures
        lProgress.text = @"Exportant factures..";
        NSString *fileFactures = [self crea_csv_factures];
        self.progres.progress = 1;
        // Si no tenim ni factures ni despeses ho indiquem i no fem res
        if (( no_factures == 0  ) &&  ( no_despeses == 0 ))
        {
            [funcComunes msg_pantalla:@"No hi ha dades per realitzar l'exportació"];
            lProgress.text = nil;
        }
        else
        {
            // Enviem el mail
            [self envia_mail:fileDespeses:fileFactures];
            lProgress.text = @"Exportació finalitzada";
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}

// Crea el CSV de factures
-(NSString *)crea_csv_factures
{
    // Fixter data
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Afegim el titol
    NSString *titol = [NSString stringWithFormat:@"%@ %@\n",@"Facturació ",trimestre_sel.txt];
    [filedata appendString:titol];
    // Construim la capçelera del fitxer
    NSMutableString *cabecera = [self set_cabecera_factures];
    // Afegim la capçelera al contingut del fitxer de sortida
    [filedata appendString:cabecera];
    // Consruim el contingut del fitxer
    NSMutableString *contingut = [self set_contingut_fitxer_factures];
    no_factures = [contingut length];
    // Si no hi han dades activem el boolean de sortida
    if ( [contingut length] == 0)
    {
        return nil;
    }
    else
    {
        no_factures = [contingut length];
        // Afegim el contingut
        [filedata appendString:contingut];
        // Escrivim el fitxer
        return [self escriu_fitxer:@"Factures.csv":filedata];
    }
}

// Crea el CSV de depeses
-(NSString *)crea_csv_despeses
{
    // Fixter data
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Afegim el titol
    NSString *titol = [NSString stringWithFormat:@"%@ %@\n",@"Despeses",trimestre_sel.txt];
    [filedata appendString:titol];
    // Construim la capçelera del fitxer
    NSMutableString *cabecera = [self set_cabecera_despeses];
    // Afegim la capçelera al contingut del fitxer de sortida
    [filedata appendString:cabecera];
    // Consruim el contingut del fitxer
    NSMutableString *contingut = [self set_contingut_fitxer_despeses];
    // Si no hi han dades activem el boolean de sortida
    no_despeses = [contingut length];
    if ( [contingut length] == 0)
    {
        return nil;
    }
    else
    {
        no_despeses = [contingut length];
        // Afegim el contingut
        [filedata appendString:contingut];
        // Escrivim el fitxer
        return [self escriu_fitxer:@"Despeses.csv":filedata];
    }
}

// Construeix el contingut del fitxer
-(NSMutableString *)set_contingut_fitxer_despeses
{
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init] ;
    [dateFormat setDateFormat:@"dd/MM/yyyy"];
    // Contingut del fitxer
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Passem per totes les despeses i trajectes ordenats per data
    // Iniciem el recorregut per tots els dies del trimestre
    NSDate *data_search = nil;
    // Linea per escriure al fitxer
    NSString *txtLineaDespesa = nil;
    // Iteracio
    int i = 0;
    // Inicialitzem la data
    data_search = trimestre_sel.datini;
    // Calculem la data final
    NSDate *lDatFin = [NSDate dateWithTimeInterval:3600*24*1 sinceDate:trimestre_sel.datfin];
    // Fem el bucle per data
    while ( [data_search compare:lDatFin] == NSOrderedAscending )
    {
        // Netejem la linea
        txtLineaDespesa = nil;
        // Obtenim els trajectes per la data
        NSMutableArray *trajectesxdata  = [funcComunes set_trajectes_by_data:data_search :managedObjectContext];
        // Tractament Trajectes
        if ( [trajectesxdata count] > 0 )
        {
            i = 0;
            NSNumber *kms = 0;
            // Sumen les Km en la mateixa data
            while ( i < [trajectesxdata count])
            {
                
                Trajecte *ltrajecte = [trajectesxdata objectAtIndex:i];
                kms = [NSNumber numberWithFloat:([kms floatValue] + [ltrajecte.km floatValue])];
                i++;
            }
            // Afegim la data
            NSString *strdata = [dateFormat stringFromDate:data_search];
            // Afegim els Kms
            // Primer calculem l'import dels kms x preu x km
            NSNumber *importTrajecte = [NSNumber numberWithFloat:([preuxKm floatValue] * [kms  floatValue])];
            // Afegim la dada
            NSString *strkms  = [NSString stringWithFormat:@",%@,%.2f €",[kms stringValue],importTrajecte.doubleValue];
            // Afegim la data, els Kms i el preu x km
            txtLineaDespesa = [strdata  stringByAppendingString:strkms];
            
            
        }
        
        // Obtenim les despeses per la data
        NSMutableArray *despesesxdata  = [funcComunes set_despeses_by_data:data_search :managedObjectContext];
        // Si tenim despeses per la data les fiquem al array de sortida del fitxer
        if ( despesesxdata != nil)
        {
            // Recuperem les categories, de les subcategories donades d'alta
            NSMutableArray *categories    =[ funcComunes setCategories:managedObjectContext];
            i = 0;
            while ( i < [categories count])
            {
                
                Categoria *lCategoria = (Categoria *)[categories objectAtIndex:i];
                // Obtenim les subcategories i per cadascuna mirem si hi ha despese
                NSMutableArray *subcategories = [funcComunes setSubCategoriesOfCategoria:lCategoria :managedObjectContext];
                
                int j = 0;
                //Boolean hi_ha = false;
                while ( j < [subcategories count])
                {
                    // Obtenim les dades de la subcategoria
                    SubCategoria *lSubCat = (SubCategoria *)[subcategories objectAtIndex:j];
                    // Obtenim si hi ha despeses categoria i subcategoria  a l'array de despeses x data
                    NSMutableArray *despeseCatSubCat = [funcComunes get_despeses_by_filter:lCategoria :lSubCat :despesesxdata];
                    // Si tenim despeses en la data / categoria / subcategoria, sumem
                    if ( [despeseCatSubCat count] > 0)
                    {
                        // Si no tenim trajectes per aquesta data, afegim el camps en blanc
                        if ( txtLineaDespesa == nil )
                        {
                            // Afegim la data
                            NSString *strdata = [dateFormat stringFromDate:data_search];
                            txtLineaDespesa = [strdata  stringByAppendingString:@",,"];
                        }
                        
                        // Sumem les depeses i afegim
                        int k= 0;
                        NSNumber *sumDespeses = 0;
                        while ( k < [despeseCatSubCat count])
                        {
                            Despesa *lDespesa = (Despesa *)[despeseCatSubCat objectAtIndex:k];
                            NSNumber *importdespesa = [NSNumber numberWithDouble:lDespesa.import.doubleValue];
                            sumDespeses = [NSNumber numberWithFloat:([sumDespeses floatValue] + [importdespesa floatValue])];
                            k++;
                        }
                        // Afegim la despesa
                        //NSString *strDespesa  = [NSString stringWithFormat:@",%@ €",[sumDespeses stringValue]];
                        NSString *strDespesa  = [NSString stringWithFormat:@",%.2f €",[sumDespeses doubleValue]];
                        
                        txtLineaDespesa = [txtLineaDespesa  stringByAppendingString:strDespesa];
                        
                    }
                    else
                    {
                        if ( txtLineaDespesa == nil )
                        {
                            // Afegim la data
                            NSString *strdata = [dateFormat stringFromDate:data_search];
                            txtLineaDespesa = [strdata  stringByAppendingString:@",,"];
                        }
                        txtLineaDespesa = [txtLineaDespesa  stringByAppendingString:@","];
                    }
                    j++;
                }
                i++;
            }
        }
        
        if ( txtLineaDespesa != nil ){
            // Afegim la linea de despesa
            txtLineaDespesa = [txtLineaDespesa stringByAppendingString:@"\n"];
            [filedata appendString:txtLineaDespesa];
            
        }
        // Incrementem la data
        data_search = [data_search dateByAddingTimeInterval:60*60*24*1];
    }
    
    return filedata;
}



// ------
// Construeix el contingut del fitxer
-(NSMutableString *)set_contingut_fitxer_factures
{
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init] ;
    [dateFormat setDateFormat:@"dd/MM/yyyy"];
    // Contingut del fitxer
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Passem per totes les factures ordenats per data
    // Iniciem el recorregut per tots els dies del trimestre
    NSDate *data_search = nil;
    // Linea per escriure al fitxer
    NSString *txtLineaFactura = nil;
    // Iteracio
    int i = 0;
    // Inicialitzem la data
    data_search = trimestre_sel.datini;
    // Calculem la data final
    NSDate *lDatFin = [NSDate dateWithTimeInterval:3600*24*1 sinceDate:trimestre_sel.datfin];
    // Fem el bucle per data
    while ( [data_search compare:lDatFin] ==  NSOrderedAscending )
    {
        // Netejem la linea
        txtLineaFactura = nil;
        // Obtenim les factures per la data
        NSMutableArray *facturesxdata  = [funcComunes set_factures_by_data:data_search :managedObjectContext];
        // Tractament factures
        if ( [facturesxdata count] > 0 )
        {
            i = 0;
            // Recorrem totes les factures que hi ha en aquesta data
            while ( i < [facturesxdata count])
            {
                
                Factura *lFactura = (Factura *)[facturesxdata objectAtIndex:i];
                // Afegim la data
                NSString *strdata = [dateFormat stringFromDate:data_search];
                txtLineaFactura = strdata;
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Tipus factura
                TipusFactura *lTipusFactura = lFactura.tipus;
                NSString *tipusFactura = lTipusFactura.txt;
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:tipusFactura];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Concepte
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:lFactura.concepte];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Nif/Cif
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:lFactura.nifcif];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Rao Social
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:lFactura.raosocial];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Base
                NSNumber *importBase  = lFactura.base;
                NSString *baseFactura = [NSString stringWithFormat:@"%.2f €", [importBase doubleValue]];
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:baseFactura];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Import amb IVA
                // Obtenim l'entitat de tipus d'iva
                TipusDIVA *LtipusIVA = lFactura.iva;
                double porcentatge   = [LtipusIVA.percent integerValue];
                double importD       = [importBase floatValue];
                float  importIva     = importD * porcentatge;
                // Dividem per 100 pel porcentatge
                importIva = importIva / 100;
                NSString *strimportIVA = [NSString stringWithFormat:@"%.2f €", importIva];
                NSNumber *importambIVA = [NSNumber numberWithFloat:([importBase floatValue] + importIva)];
                NSString *strimportambIVA = [NSString stringWithFormat:@"%.2f €", [importambIVA doubleValue]];
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:strimportambIVA];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Tipus d'IVA
                NSString *tipusDIVA = LtipusIVA.txt;
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:tipusDIVA];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // Percentatge
                NSString *percentatgeDIVA = [NSString stringWithFormat:@"%@", LtipusIVA.percent ];
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:percentatgeDIVA];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@","];
                // IVA
                txtLineaFactura = [txtLineaFactura  stringByAppendingString:strimportIVA];
                txtLineaFactura = [txtLineaFactura stringByAppendingString:@"\n"];
                // Escrivim
                [filedata appendString:txtLineaFactura];
                
                i++;
            }
        }
        // Incrementem la data
        data_search = [data_search dateByAddingTimeInterval:60*60*24*1];
    }
    
    return filedata;
}

// ------

// Escriu el fitxer
-(NSString *)escriu_fitxer:(NSString *)fitxer
                          :(NSMutableString *)filedata
{
    
    // Control error
    NSError *csvError = NULL;
    
    // Fitxer de sortida
    NSString *outputFileName = [docDirectory stringByAppendingPathComponent:fitxer];
    
    // Escrivim al fitxer i controlem error
    BOOL written = [filedata writeToFile:outputFileName atomically:YES encoding:NSUTF8StringEncoding error:&csvError];
    
    
    
    //If there was a problem saving we show the error if not show success and file path
    if (!written) [funcComunes msg_pantalla:[NSString stringWithFormat:@"%@,%@,%@",@"Fitxer ",fitxer,@"generat correctament"]];
    
    return outputFileName;
}

// Construeix la capcelera del fitxer de depeses
-(NSMutableString *)set_cabecera_despeses
{
    // Contingut del fitxer
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Recuperem les categories, de les subcategories donades d'alta
    NSMutableArray *categories    =[ funcComunes setCategories:managedObjectContext];
    // Capçeleres
    NSString *cab_primer_nivell = nil;
    NSString *cab_segon_nivell = nil;
    // Fixem la primera part de la capçelera
    // Data  Km  Preu x Km  Categoria1        Categoria2 .....
    //                      SubCat1 SubCa2    SubCat1 SubCat2 ....
    cab_primer_nivell = @"Data,Kms, Import Trajecte";
    cab_segon_nivell = @",,";
    // Afegim les categories i les seves subcategories
    // Per cada categoria, mirem si tenim subcategories i pel parell si hi ha despeses
    int i = 0;
    
    while ( i < [categories count])
    {
        
        Categoria *lCategoria = (Categoria *)[categories objectAtIndex:i];
        // Obtenim les subcategories i per cadascuna mirem si hi ha despese
        NSMutableArray *subcategories = [funcComunes setSubCategoriesOfCategoria:lCategoria :managedObjectContext];
        NSString *txtCategoria = nil;
        int j = 0;
        //Boolean hi_ha = false;
        while ( j < [subcategories count])
        {
            
            SubCategoria *lSubCat = (SubCategoria *)[subcategories objectAtIndex:j];
            if ( txtCategoria == nil )
            {
                txtCategoria = [@"," stringByAppendingString:lCategoria.txt];
                cab_primer_nivell = [cab_primer_nivell stringByAppendingString:txtCategoria];
            } else cab_primer_nivell = [cab_primer_nivell stringByAppendingString:@","];
            NSString *txtSubcategoria =  [@"," stringByAppendingString: lSubCat.txt];
            cab_segon_nivell = [cab_segon_nivell stringByAppendingString:txtSubcategoria];
            j++;
        }
        i++;
    }
    // Una vegada tenim les capceleres, afegim el salt de linea;
    cab_primer_nivell = [cab_primer_nivell stringByAppendingString:@"\n"];
    cab_segon_nivell  = [cab_segon_nivell stringByAppendingString:@"\n"];
    // Construim les dades de sortida
    [filedata appendString:cab_primer_nivell];
    [filedata appendString:cab_segon_nivell];
    return filedata;
}


// Construeix la capcelera del fitxer de factures
-(NSMutableString *)set_cabecera_factures
{
    // Contingut del fitxer
    NSMutableString *filedata = [NSMutableString stringWithCapacity:0];
    // Capçelera
    NSString *cab_primer_nivell = nil;
    // Fixem la primera part de la capçelera
    // Data Tipus de factura Concepte NIF/CIF Rao social Base   Import (amb IVA) Import (sense IVA) IVA
    cab_primer_nivell = @"Data, Tipus de factura, Concepte, NIF/CIF, Rao social, Base ,  Import (amb IVA), Tipus d'IVA, Percentatge, Import d'IVA";
    // Una vegada tenim les capceleres, afegim el salt de linea;
    cab_primer_nivell = [cab_primer_nivell stringByAppendingString:@"\n"];
    // Construim les dades de sortida
    [filedata appendString:cab_primer_nivell];
    return filedata;
}





- (void) mailComposeController:(MFMailComposeViewController *)controller didFinishWithResult:(MFMailComposeResult)result error:(NSError *)error
{
    @try {
        
        switch (result)
        {
            case MFMailComposeResultCancelled:
                NSLog(@"Mail cancelled");
                break;
            case MFMailComposeResultSaved:
                NSLog(@"Mail saved");
                break;
            case MFMailComposeResultSent:
                NSLog(@"Mail sent");
                break;
            case MFMailComposeResultFailed:
                NSLog(@"Mail sent failure: %@", [error localizedDescription]);
                break;
            default:
                break;
        }
        
        // Close the Mail Interface
        [self dismissViewControllerAnimated:YES completion:NULL];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}



// Envia el mail
-(void)envia_mail:(NSString *)fileDespeses
                 :(NSString *)fileFactures
{
    
    // Mail
    NSArray *toRecipents = nil;
    // Titol del mail
    NSString *emailTitle = [NSString stringWithFormat:@"%@ %@",@"Control despeses ",trimestre_sel.txt];
    // Contingut
    NSString *messageBody = [NSString stringWithFormat:@"%@ %@",@"Annexat es troben les despeses i factures del ",trimestre_sel.txt];
    // Receptor del mail, serà configurable.....
    if ( [MailArray count] > 0 )
    {
        Mail *lMail = [MailArray objectAtIndex:0];
        toRecipents = [NSArray arrayWithObject:lMail.mail];
    }
    // Construim el mail
    MFMailComposeViewController *mc = [[MFMailComposeViewController alloc] init];
    mc.mailComposeDelegate = self;
    [mc setSubject:emailTitle];
    [mc setMessageBody:messageBody isHTML:NO];
    [mc setToRecipients:toRecipents];
    // Afegim l'annexe de despeses
    [mc addAttachmentData:[NSData dataWithContentsOfFile:fileDespeses]
                 mimeType:@"text/csv"
                 fileName:@"Despeses.csv"];
    [mc addAttachmentData:[NSData dataWithContentsOfFile:fileFactures]
                 mimeType:@"text/csv"
                 fileName:@"Factures.csv"];
    // Present mail view controller on screen
    [self presentViewController:mc animated:YES completion:NULL];
    
    
}


@end
