//
//  ClScrGestioDespeses.m
//  TaxControl
//
//  Created by Javier Cava García on 12/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrGestioDespeses.h"
#import "ClComunes.h"
#import "Despesa.h"
#import "ClScrDespeses.h"



@interface ClScrGestioDespeses ()

@property (strong, nonatomic) NSMutableDictionary *seccions;
@property (strong, nonatomic) NSArray *sortedData;

@end


@implementation ClScrGestioDespeses
{
    ClComunes *funcComunes;
 
}


// Array de Despeses
@synthesize DespesesArray;

// Core data
@synthesize managedObjectContext;
// Dades
@synthesize iniDate;
@synthesize fiDate;
@synthesize lblDatfin;
@synthesize lblDatini;

@synthesize seccions;
@synthesize sortedData;


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    @try {
        // Establim les despeses
        NSMutableArray *mutableFetchResults  = [funcComunes set_despeses:iniDate :fiDate :managedObjectContext];
        [self setDespesesArray:mutableFetchResults];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblDespeses addGestureRecognizer:tapEdit];
        // Establim les seccions del table view, segons la categoria de la despesa
        [self set_seccions_despeses];
        [_tblDespeses reloadData];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Estableix les seccions
-(void)set_seccions_despeses
{
    seccions = nil;
    seccions = [NSMutableDictionary dictionary];
    for (Despesa *despesa in DespesesArray)
    {
        NSMutableArray *categoriesaSeccio = [self.seccions objectForKey:despesa.subcategoria.categoria.txt];
        if (categoriesaSeccio == nil) {
            
            categoriesaSeccio = [NSMutableArray array];
            [self.seccions setObject:categoriesaSeccio forKey:despesa.subcategoria.categoria.txt];
        }
        // Afegim la despesa dins la seccio
        [categoriesaSeccio addObject:despesa];
    }
    // Create a sorted list
    NSArray *arrayCat = [self.seccions allKeys];
    self.sortedData = [arrayCat sortedArrayUsingSelector:@selector(compare:)];
    
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [self.seccions count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSString *txtCategoria = [self.sortedData objectAtIndex:section];
    NSArray *categoriesOnData = [self.seccions objectForKey:txtCategoria];
    return [categoriesOnData count];
}

// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        CGPoint tapLocation = [cell locationInView:self.tblDespeses];
        NSIndexPath *indexPath = [self.tblDespeses indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de despeses
            ClScrDespeses *editDespeses = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewDespeses"];
            // Activem el mode d'edicio
            editDespeses.edicio = @1;
            // Obtenim el id de la categoria a editar
            // Controla SECCIO !!!!!!!!
            NSString *txtCategoria    = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsCategoria = [self.seccions objectForKey:txtCategoria];
            Despesa *lDespesa       = [itemsCategoria objectAtIndex:indexPath.row];
            editDespeses.DespesaEdit = lDespesa;
            editDespeses.navigationItem.prompt = @"Edició Despesa";
            // Cridem la pantalla
            [self.navigationController pushViewController:editDespeses animated:YES];
        }
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
  
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    // Configurem les cells
    NSString *txtCategoria       = [self.sortedData objectAtIndex:indexPath.section];
    NSArray  *itemsCategoria    = [self.seccions objectForKey:txtCategoria];
    Despesa *lDespesa          = [itemsCategoria objectAtIndex:indexPath.row];
    UILabel  *labelCatSubcat    = (UILabel *)[cell viewWithTag:1];
    UILabel  *labelData         = (UILabel *)[cell viewWithTag:2];
    UILabel  *labelimport       = (UILabel *)[cell viewWithTag:3];
    UILabel  *labelConcepte     = (UILabel *)[cell viewWithTag:4];
    UILabel  *labelIva          = (UILabel *)[cell viewWithTag:5];
    UILabel  *labelTIva         = (UILabel *)[cell viewWithTag:6];
    
    // Format data
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"dd/MM/yyyy"];
    // Establim textes
    SubCategoria *lSubCategoria = lDespesa.subcategoria;
    labelCatSubcat.text = [NSString stringWithFormat:@"%@", lSubCategoria.txt];
    labelData.text = [NSString stringWithFormat:@"%@",[formatter stringFromDate: lDespesa.data]];
    // Convertim a String
    // Formatejem l'import
    NSNumber *importDespesa = lDespesa.import;
    NSString *strImport     = [NSString stringWithFormat:@"%.2f €", [importDespesa doubleValue]];
    labelimport.text        = [NSString stringWithFormat:@"Import : %@",strImport];
    labelConcepte.text      = [NSString stringWithFormat:@"%@",lDespesa.concepte];
    // Import IVA
    // Obtenim l'entitat de la subcategoria per treure l'iva
    // Amb la subcategoria ya tenim el tipus d'iva, obtenim la seva entitat per informar la resta de camps
    TipusDIVA *LtipusIVA = lSubCategoria.iva;
    // Calculem l'import IVA
    double porcentatge   = [LtipusIVA.percent integerValue];
    double importD       = [importDespesa floatValue];
    double importIva     = importD * porcentatge;
    // Dividem per 100 pel porcentatge
    importIva = importIva / 100;
    // Convertim a String
    NSString *strImportIva = [NSString stringWithFormat:@"IVA : %.2f €", importIva];
    // Informem els camps
    labelIva.text = strImportIva;
    // Tipus d'IVA
    labelTIva.text = LtipusIVA.txt;
    return cell;
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
 
    NSString *txtCategoria = [self.sortedData objectAtIndex:section];
    return txtCategoria;
}
// Definició del text de les seccions
-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    UILabel *customLabel        = [[UILabel alloc] init];
    customLabel.text            = [self tableView:tableView titleForHeaderInSection:section];
    customLabel.textColor       = [UIColor blackColor];
    customLabel.backgroundColor = [UIColor lightGrayColor];
    return customLabel;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}



// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    @try {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            NSError *error = nil;
            NSString *txtCategoria    = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsCategoria = [self.seccions objectForKey:txtCategoria];
            Despesa *lDespesa       = [itemsCategoria objectAtIndex:indexPath.row];
            //Despeses *lDespesa  = (Despeses *)[DespesesArray  objectAtIndex: indexPath.row];
            [managedObjectContext deleteObject:lDespesa];
            [managedObjectContext save:&error];
            // Elimina l'entrada de l'array
            [DespesesArray removeObject:lDespesa];
            // Establim les seccions del table view, segons la categoria de la despes
            [self set_seccions_despeses];
            // Elimina la fila del tableview
            //[tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
            [_tblDespeses reloadData];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}






@end
