//
//  ClScrGestioFactures.m
//  TaxControl
//
//  Created by Javier Cava García on 12/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrGestioFactures.h"
#import "ClComunes.h"
#import "Factura.h"
#import "ClScrFacturacio.h"



@interface ClScrGestioFactures ()

@property (strong, nonatomic) NSMutableDictionary *seccions;
@property (strong, nonatomic) NSArray *sortedData;

@end


@implementation ClScrGestioFactures
{
    ClComunes *funcComunes;
 
}


// Array de Factures
@synthesize FacturesArray;

// Core data
@synthesize managedObjectContext;
// Dades
@synthesize iniDate;
@synthesize fiDate;
@synthesize lblDatfin;
@synthesize lblDatini;

@synthesize seccions;
@synthesize sortedData;


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    @try {
        // Establim les factures
        NSMutableArray *mutableFetchResults  = [funcComunes set_factures:iniDate :fiDate :managedObjectContext];
        [self setFacturesArray:mutableFetchResults];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblFactures addGestureRecognizer:tapEdit];
        // Establim les seccions del table view, segons la categoria de la despesa
        [self set_seccions_factures];
        [_tblFactures reloadData];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Estableix les seccions
-(void)set_seccions_factures
{
    seccions = nil;
    seccions = [NSMutableDictionary dictionary];
    for (Factura *factura in FacturesArray)
    {
        NSMutableArray *tipusSeccio = [self.seccions objectForKey:factura.tipus.txt];
        if (tipusSeccio == nil) {
            
            tipusSeccio = [NSMutableArray array];
            [self.seccions setObject:tipusSeccio forKey:factura.tipus.txt];
        }
        // Afegim la despesa dins la seccio
        [tipusSeccio addObject:factura];
    }
    // Create a sorted list
    NSArray *arrayCat = [self.seccions allKeys];
    self.sortedData = [arrayCat sortedArrayUsingSelector:@selector(compare:)];
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [self.seccions count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSString *txtTipusFactura = [self.sortedData objectAtIndex:section];
    NSArray *itembyTipus = [self.seccions objectForKey:txtTipusFactura];
    return [itembyTipus count];
}

// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        CGPoint tapLocation = [cell locationInView:self.tblFactures];
        NSIndexPath *indexPath = [self.tblFactures indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de despeses
            ClScrFacturacio *editFactura = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewFactures"];
            // Activem el mode d'edicio
            editFactura.edicio = @1;
            // Obtenim el id de la categoria a editar
            // Controla SECCIO !!!!!!!!
            NSNumber *idTipusFactura = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsFactures  = [self.seccions objectForKey:idTipusFactura];
            Factura *lFactura       = [itemsFactures objectAtIndex:indexPath.row];
            editFactura.FacturaEdit = lFactura;
            editFactura.navigationItem.prompt = @"Edició Factura";
            // Cridem la pantalla
            [self.navigationController pushViewController:editFactura animated:YES];
        }
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
  
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    // Configurem les cells
    NSNumber *idTipusFactura    = [self.sortedData objectAtIndex:indexPath.section];
    NSArray  *itemsTipusFactura = [self.seccions objectForKey:idTipusFactura];
    Factura *lFactura          = [itemsTipusFactura objectAtIndex:indexPath.row];
    
    UILabel  *labelConcepte     = (UILabel *)[cell viewWithTag:1];
    UILabel  *labelRao          = (UILabel *)[cell viewWithTag:2];
    UILabel  *labelBase         = (UILabel *)[cell viewWithTag:3];
    UILabel  *labelData         = (UILabel *)[cell viewWithTag:4];
    UILabel  *labelIva          = (UILabel *)[cell viewWithTag:5];
    UILabel  *labelCIF          = (UILabel *)[cell viewWithTag:6];
    UILabel  *labelTIva         = (UILabel *)[cell viewWithTag:7];
    
    
    
    
    // Format data
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"dd/MM/yyyy"];
    // Establim textes
    labelConcepte.text  = [NSString stringWithFormat:@"%@",lFactura.concepte];
    labelRao.text       = [NSString stringWithFormat:@"%@",lFactura.raosocial];
    labelData.text      = [formatter stringFromDate: lFactura.data];
    // Convertim a String
    // Formatejem l'import
    NSNumber *importBase = lFactura.base;
    NSString *strImport = [NSString stringWithFormat:@"Base : %.2f €", [importBase doubleValue]];
    labelBase.text = strImport;
    // CIF /NIF
    labelCIF.text = [NSString stringWithFormat:@"%@",lFactura.nifcif];
    // IVA
    // Obtenim l'entitat de tipus d'iva
    TipusDIVA *LtipusIVA = lFactura.iva;
    double porcentatge   = [LtipusIVA.percent integerValue];
    double importD       = [importBase floatValue];
    double importIva     = importD * porcentatge;
    // Dividem per 100 pel porcentatge
    importIva = importIva / 100;
    // Convertim a String
    NSString *strImportIva = [NSString stringWithFormat:@"IVA : %.2f €", importIva];
    // Informem els camps
    labelIva.text = strImportIva;
    // Tipus d'IVA
    labelTIva.text = LtipusIVA.txt;
    return cell;
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    NSString *txtTipusFactura = [self.sortedData objectAtIndex:section];
    return txtTipusFactura;
}

// Definició del text de les seccions
-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    UILabel *customLabel        = [[UILabel alloc] init];
    customLabel.text            = [self tableView:tableView titleForHeaderInSection:section];
    customLabel.textColor       = [UIColor blackColor];
    customLabel.backgroundColor = [UIColor lightGrayColor];
    return customLabel;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}



// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    @try {
        
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            NSError *error = nil;
            NSNumber *idTipusFactura    = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsTipusFactura = [self.seccions objectForKey:idTipusFactura];
            Factura *lFactura          = [itemsTipusFactura objectAtIndex:indexPath.row];
            [managedObjectContext deleteObject:lFactura];
            [managedObjectContext save:&error];
            // Elimina l'entrada de l'array
            [FacturesArray removeObject:lFactura];
            // Establim les seccions del table view, segons la categoria de la despes
            [self set_seccions_factures];
            [_tblFactures reloadData];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}



// Liberamos memoria
-(void)viewWillDisappear:(BOOL)animated
{
   
}


@end
