//
//  ClScrGestioTrajectes.m
//  TaxControl
//
//  Created by Javier Cava García on 12/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrGestioTrajectes.h"
#import "ClComunes.h"
#import "Trajecte.h"
#import "ClScrTrajectes.h"



@interface ClScrGestioTrajectes()

@property (strong, nonatomic) NSMutableDictionary *seccions;
@property (strong, nonatomic) NSArray *sortedData;

@end


@implementation ClScrGestioTrajectes
{
    ClComunes *funcComunes;
 
}


// Array de Factures
@synthesize TrajectesArray;

// Core data
@synthesize managedObjectContext;
// Dades
@synthesize iniDate;
@synthesize fiDate;
@synthesize lblDatfin;
@synthesize lblDatini;

@synthesize seccions;
@synthesize sortedData;



- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    @try {
        // Establim els trajectes
        NSMutableArray *mutableFetchResults  = [funcComunes set_trajectes:iniDate :fiDate :managedObjectContext];
        [self setTrajectesArray:mutableFetchResults];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblTrajectes addGestureRecognizer:tapEdit];
        // Establim les seccions del table view, segons la categoria de la despesa
        [self set_seccions_trajectes];
        [_tblTrajectes reloadData];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Estableix les seccions
-(void)set_seccions_trajectes
{
    seccions = nil;
    seccions = [NSMutableDictionary dictionary];
    for (Trajecte *trajecte in TrajectesArray)
    {
        NSMutableArray *tipusSeccio = [self.seccions objectForKey:trajecte.data];
        if (tipusSeccio == nil) {
            
            tipusSeccio = [NSMutableArray array];
            [self.seccions setObject:tipusSeccio forKey:trajecte.data];
        }
        // Afegim la despesa dins la seccio
        [tipusSeccio addObject:trajecte];
    }
    // Create a sorted list
    NSArray *arrayTrajectes = [self.seccions allKeys];
    self.sortedData = [arrayTrajectes sortedArrayUsingSelector:@selector(compare:)];
    
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [self.seccions count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    NSDate  *idDataTrajecte = [self.sortedData objectAtIndex:section];
    NSArray *itembyData = [self.seccions objectForKey:idDataTrajecte];
    return [itembyData count];
}

// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        CGPoint tapLocation = [cell locationInView:self.tblTrajectes];
        NSIndexPath *indexPath = [self.tblTrajectes indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de despeses
            ClScrTrajectes *editTrajecte = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewTrajectes"];
            // Activem el mode d'edicio
            editTrajecte.edicio = @1;
            // Obtenim el id de la categoria a editar
            // Controla SECCIO !!!!!!!!
            NSNumber *idDataTrajecte = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsdata      = [self.seccions objectForKey:idDataTrajecte];
            Trajecte *lTrajecte       = [itemsdata objectAtIndex:indexPath.row];
            editTrajecte.TrajecteEdit = lTrajecte;
            editTrajecte.navigationItem.prompt = @"Edició Trajecte";
            // Cridem la pantalla
            [self.navigationController pushViewController:editTrajecte animated:YES];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}



- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
  
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    // Configurem les cells
    NSDate *idDataTrajecte    = [self.sortedData objectAtIndex:indexPath.section];
    NSArray  *itemsData         = [self.seccions objectForKey:idDataTrajecte];
    Trajecte *lTrajecte         = [itemsData objectAtIndex:indexPath.row];
    
    UILabel  *labelOrigen = (UILabel *)[cell viewWithTag:1];
    UILabel  *labelDesti  = (UILabel *)[cell viewWithTag:2];
    UILabel  *labelkm     = (UILabel *)[cell viewWithTag:3];
    
    // Establim textes
    labelOrigen.text    = [NSString stringWithFormat:@"Origen : %@",lTrajecte.origen];
    labelDesti.text     = [NSString stringWithFormat:@"Destinació : %@",lTrajecte.desti];
    // Convertim a String
    // Formatejem l'import
    NSNumber *kmTrajecte = lTrajecte.km;
    NSString *strkm = [NSString stringWithFormat:@"%.2f Km", [kmTrajecte doubleValue]];
    labelkm.text = strkm;
    return cell;
}


- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    // Format data
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:@"dd MMMM , yyyy"];
    // Establim la localització perque surti en català
    formatter.locale = [[NSLocale alloc] initWithLocaleIdentifier:@"CA"];
    NSDate  *idDataTrajecte = [self.sortedData objectAtIndex:section];
    
    return [formatter stringFromDate: idDataTrajecte];
    
}

// Definició del text de les seccions
-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    UILabel *customLabel        = [[UILabel alloc] init];
    customLabel.text            = [self tableView:tableView titleForHeaderInSection:section];
    customLabel.textColor       = [UIColor blackColor];
    customLabel.backgroundColor = [UIColor lightGrayColor];
    return customLabel;
}


// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}



// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    @try {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            NSError *error = nil;
            NSDate   *idDataTrajecte    = [self.sortedData objectAtIndex:indexPath.section];
            NSArray  *itemsData         = [self.seccions objectForKey:idDataTrajecte];
            Trajecte *lTrajecte         = [itemsData objectAtIndex:indexPath.row];
            [managedObjectContext deleteObject:lTrajecte];
            [managedObjectContext save:&error];
            // Elimina l'entrada de l'array
            [TrajectesArray removeObject:lTrajecte];
            // Establim les seccions del table view, segons la categoria de la despes
            [self set_seccions_trajectes];
            [_tblTrajectes reloadData];
        }
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}







@end
