//
//  ClScrNovaSubcategoria.m
//  TaxControl
//
//  Created by Javier Cava García on 14/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrNovaSubcategoria.h"
#import "SubCategoria.h"
#import "TipusDIVA.h"
#import "Categoria.h"
#import "ClComunes.h"

@interface ClScrNovaSubcategoria ()
{
    ClComunes *funcComunes;
}
@end

@implementation ClScrNovaSubcategoria

// Array de Categories
@synthesize CategoriesArray;
// Array de Tipus d'IVA
@synthesize TipusDIVAArray;
// Control Core data
@synthesize managedObjectContext;
// Stepper Categoria
@synthesize stepCategoria;
@synthesize txtCategoria;
@synthesize txtSubcategoria;
@synthesize txtTipusdiva;
@synthesize SubCategoriaEdit;
@synthesize edicio;
@synthesize btngravar;



- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Establim el fons
        self.view.backgroundColor = [[UIColor alloc] initWithPatternImage:[UIImage       imageNamed:@"Background.png"]];
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        // Carreguem els tipus d'IVA
        NSMutableArray *mutableFetchResults = [funcComunes setTipusDIVA:managedObjectContext];
        [self setTipusDIVAArray:mutableFetchResults];
        mutableFetchResults = nil;
        
        // Si estem en mode edicio carreguem la categoria
        if ( [edicio  isEqual: @1] )
        {
            // Recuperem la categoria segons l'ID
            Categoria *lCategoria = SubCategoriaEdit.categoria;
            txtCategoria.text     = lCategoria.txt;
            txtSubcategoria.text  = SubCategoriaEdit.txt;
            TipusDIVA *lIva       = SubCategoriaEdit.iva;
            txtTipusdiva.text     = lIva.txt;
            
            // Si no tenim tipus d'iva no podem crear subcategories
            if ( TipusDIVAArray == nil )
            {
                // Misstage
                [funcComunes msg_pantalla: @"Imposible crear subcategories, es nessaria la definició del tipus d'IVA!"];
                // Desactivem el botó gravar
                btngravar.enabled = false;
            }
            
        }
        else
        {
            // Carreguem les categories
            mutableFetchResults  = [funcComunes setCategories:managedObjectContext];
            [self setCategoriesArray:mutableFetchResults];
            mutableFetchResults = nil;
            // Si no tenim categories o tipus d'iva no podem crear subcategories
            if (( CategoriesArray == nil ) ||  ( TipusDIVAArray == nil ))
            {
                // Misstage
                [funcComunes msg_pantalla: @"Imposible crear subcategories, configuració necessaria!"];
                // Desactivem el botó gravar
                btngravar.enabled = false;
            }
            
        }
        
        [txtSubcategoria resignFirstResponder];
        // Configuren el stepper de categories
        [self configura_step_categoria];
        // Configuren el stepper de tipus d'IVA
        [self configura_step_tipusdiva];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
    

}

// Configura step categories
-(void)configura_step_categoria
{
    [self.stepCategoria setStepValue: 1];
    [self.stepCategoria setMinimumValue:0];
    if ( CategoriesArray == nil)
    {
        [self.stepCategoria setMaximumValue:0];
    }else [self.stepCategoria setMaximumValue:[CategoriesArray count]-1];
    // Inicialitzem el text de la categoria
    if ( [txtCategoria text].length == 0)
    {
        NSUInteger ini = 0;
        [self setvalorcategoria:&ini];
    }
    
}

// Configura step tipus d'IVA
-(void)configura_step_tipusdiva
{
    [self.stepTipusdiva setStepValue: 1];
    [self.stepTipusdiva setMinimumValue:0];
    [self.stepTipusdiva setMaximumValue:[TipusDIVAArray count]-1];
    // Inicialitzem el text de la categoria
    if ( [txtTipusdiva text].length == 0)
    {
        NSUInteger ini = 0;
        [self setvalortipusdiva:&ini];
    }
    
}

// Estableix el valor de la categoria en funció de l'index d'entrada
// i l'array de categories
-(void)setvalorcategoria:(NSUInteger *)index
{
    
    Categoria *lCat  = (Categoria *)[CategoriesArray   objectAtIndex:  *index];
    txtCategoria.text = lCat.txt;
   
}

// Estableix el valor del tipus d'IVA en funció de l'index d'entrada
// i l'array de categories
-(void)setvalortipusdiva:(NSUInteger *)index
{
    
    TipusDIVA *lTipusdiva  = (TipusDIVA *)[TipusDIVAArray   objectAtIndex:  *index];
    txtTipusdiva.text = lTipusdiva.txt;
    
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


// Grabació Nova Subcategoria
- (IBAction)btnSave:(UIButton *)sender
{
    
    NSError *error = nil;
    @try {
        // Unicament grabem si s'ha introduit el texte de la categoria
        if ( [txtSubcategoria text].length == 0 )
        {
            [funcComunes msg_pantalla: @"Indiqui la descripció de la nova subcategoria"];
        }else
        {
            
            if ( [edicio  isEqual: @1] )
            {
                [SubCategoriaEdit setTxt:[txtSubcategoria text]];
                TipusDIVA *lIva = [TipusDIVAArray objectAtIndex:self.stepTipusdiva.value];
                SubCategoriaEdit.iva = lIva;
            }
            else
            {
                // Obtenim la categoria
                Categoria *lCat = (Categoria *) [CategoriesArray objectAtIndex:self.stepCategoria.value];
                // Obtenim el tipus d'IVA
                TipusDIVA *lTipusDIVA = (TipusDIVA *) [TipusDIVAArray objectAtIndex:self.stepTipusdiva.value];
                // Chequejem que la categoria no existeixi
                SubCategoria *lSubCategoria = [funcComunes get_Subcategoria_by_txt:lCat:lTipusDIVA :txtSubcategoria.text:managedObjectContext];
                if ( lSubCategoria != nil)
                {
                    // La SubCategoria ja existeix, no deixem grabar
                    [funcComunes msg_pantalla:@"Ya existeix aquesta Subcategoria!"];
                    return;
                }
                
                SubCategoria *lSubCat = [NSEntityDescription
                                         insertNewObjectForEntityForName:@"SubCategoria"
                                         inManagedObjectContext:managedObjectContext];
                [lSubCat setTxt:[txtSubcategoria text]];
                lSubCat.categoria = lCat;
                lSubCat.iva = lTipusDIVA;
                //Netejem el text
                txtSubcategoria.text = nil;
            }
            // Save
            if (![managedObjectContext save:&error]) {
                // Handle the error.
                [funcComunes msg_pantalla: @"Error al gravar la Subcategoria"];
            }else
            {
                if ( [edicio  isEqual: @1] )
                {
                    //Mostrem misstatge
                    [funcComunes msg_pantalla:@"Subcategoria actualitzada correctament"];
                    
                } else
                    //Mostrem misstatge
                    [funcComunes msg_pantalla:@"Subcategoria creada correctament"];
            }
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}



// Retorna l'ultim ID
-(NSNumber *)  GetLastIDSubCategoria
{
    
    SubCategoria *lSubCat = (SubCategoria *) [NSEntityDescription
                                     entityForName:@"SubCategoria"
                                     inManagedObjectContext:managedObjectContext];
    
    NSFetchRequest *Request;
    NSEntityDescription *entity;
    NSArray *seleccion;
    NSError *error;
    NSNumber *linival = [[NSNumber alloc] initWithInt:1];
    NSNumber *ultid;
    // Recuperem les categories
    // Establim l'ordre
    NSSortDescriptor *sort = [[NSSortDescriptor alloc] initWithKey:@"id" ascending:YES];
    // Inicilitzem el request
    Request    = [[NSFetchRequest alloc] init];
    // Inicialitzem la entitat
    entity = [[NSEntityDescription alloc] init];
    entity = [NSEntityDescription entityForName:@"SubCategoria" inManagedObjectContext:managedObjectContext];
    // Establim les propietats del request
    [Request setEntity:entity];
    [Request setSortDescriptors:@[sort]];
    // Fem la select
    seleccion  = [managedObjectContext executeFetchRequest:Request error:&error];
    
    if ([seleccion count] > 0 )
    {
        for ( int i = 0; i < [seleccion count]; i++)
        {
            lSubCat = [seleccion  objectAtIndex:i];
        }
        //linival = lSubCat.id;
        double valor = [linival doubleValue];
        ultid = [NSNumber numberWithDouble: valor + 1];
        return ultid;
    } else return linival;
}



// Canvi de valor a step categoria
- (IBAction)stepCategoriaChangeValue:(UIStepper *)sender {
    
    NSUInteger stepvalue = [sender value];
    if ( ( stepvalue <= [CategoriesArray count]-1 ))
    [self setvalorcategoria:&stepvalue];
}
// Canvi de valor a step tipus d'IVA
- (IBAction)stepTipusdivaChangeValue:(UIStepper *)sender {
    
    NSUInteger stepvalue = [sender value];
    if ( ( stepvalue <= [TipusDIVAArray count]-1 ))
        [self setvalortipusdiva:&stepvalue];
}

// Liberamos memoria
-(void)viewWillDisappear:(BOOL)animated
{

    
 }

// Ocultar teclat
-(IBAction) txtDoneEditing : (id) sender{
    [sender resignFirstResponder];
}

@end
