//
//  ClScrSubCategories.m
//  TaxControl
//
//  Created by Javier Cava García on 14/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrSubCategories.h"
#import "Categoria.h"
#import "ClComunes.h"
#import "ClScrNovaSubcategoria.h"

@interface ClScrSubCategories ()

@end

@implementation ClScrSubCategories
{
    ClComunes *funcComunes;
}
// Array de Tipus d'IVA
@synthesize TipusDIVAArray;
// Array de Categories
@synthesize CategoriesArray;
// Array de Subcategories
@synthesize SubCategoriesArray;
// Core data
@synthesize managedObjectContext;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    
    NSMutableArray *mutableFetchResults;
    @try {
        // Carreguem les categories
        mutableFetchResults  = [funcComunes setCategories:managedObjectContext];
        [self setCategoriesArray:mutableFetchResults];
        mutableFetchResults = nil;
        // Carreguem les subcategories
        mutableFetchResults  = [funcComunes setSubCategories:managedObjectContext];
        [self setSubCategoriesArray:mutableFetchResults];
        [_tblSubCategories reloadData];
        mutableFetchResults = nil;
        // Carreguem els tipus d'IVA
        mutableFetchResults  = [funcComunes setTipusDIVA:managedObjectContext];
        [self setTipusDIVAArray:mutableFetchResults];
        mutableFetchResults = nil;
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }

}

- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblSubCategories addGestureRecognizer:tapEdit];
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        CGPoint tapLocation = [cell locationInView:self.tblSubCategories];
        NSIndexPath *indexPath = [self.tblSubCategories indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de nova categoria pero en mode edicio
            ClScrNovaSubcategoria *editSubCatView = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewSubCategoria"];
            // Activem el mode d'edicio
            editSubCatView.edicio = @1;
            // Obtenim el id de la categoria a editar
            SubCategoria *lSubcategoria  = (SubCategoria *)[SubCategoriesArray  objectAtIndex: indexPath.row];
            editSubCatView.SubCategoriaEdit = lSubcategoria;
            editSubCatView.navigationItem.prompt = @"Edició Subcategoria";
            // Cridem la pantalla
            [self.navigationController pushViewController:editSubCatView animated:YES];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [SubCategoriesArray count];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    SubCategoria *lSubCat  = (SubCategoria *)[SubCategoriesArray  objectAtIndex: indexPath.row];
    // Obtenim el text de la categoria
    Categoria *LCategoria = lSubCat.categoria;
    NSString *txtCat = LCategoria.txt;
    // Establim el text de la Cell
    UILabel *txtLabel = (UILabel *)[cell viewWithTag:1];
    txtLabel.text = txtCat;
    // Establim el detall de la cell
    UILabel  *txtDetail = (UILabel *)[cell viewWithTag:2];
    txtDetail.text = [lSubCat txt];
    // Establim l'IVA de la subcategoria
    UILabel  *txtIVA = (UILabel *)[cell viewWithTag:3];
    TipusDIVA *lIVA = lSubCat.iva;
    txtIVA.text = lIVA.txt;
    return cell; 
}



// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSError *error;
    @try {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            SubCategoria *lSubCat  = (SubCategoria *)[SubCategoriesArray  objectAtIndex: indexPath.row];
            // Abans d'eliminar, comprobem integritat referencial
            NSMutableArray *ref = [funcComunes get_references_Subcategoria:lSubCat :managedObjectContext];
            if ( [ref  count] > 0)
            {
                [funcComunes msg_pantalla:@"La Subcategoria encara s'utilitza, no es pot eliminar!"];
            }
            else
            {
                [managedObjectContext deleteObject:lSubCat];
                [managedObjectContext save:&error];
                // Elimina l'entrada de l'array
                [SubCategoriesArray removeObjectAtIndex:indexPath.row];
                // Elimina la fila del tableview
                [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
                [_tblSubCategories reloadData];
            }
        }
        else if (editingStyle == UITableViewCellEditingStyleInsert) {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }
        
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}


@end
