//
//  ClScrTipusFactures.m
//  TaxControl
//
//  Created by Javier Cava García on 14/10/13.
//  Copyright (c) 2013 Javier Cava García. All rights reserved.
//

#import "ClScrTipusFactures.h"
#import "ClComunes.h"
#import "ClScrNouTipusFactura.h"

@interface ClScrTipusFactures ()

@end

@implementation ClScrTipusFactures
{
    ClComunes *funcComunes;
}

// Array de Tipus de factures
@synthesize TipusFacturesArray;
// Core data
@synthesize managedObjectContext;

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

// Carreguem dades cada vegada que es mostra el tableView
-(void)viewWillAppear:(BOOL)animated
{
    
    @try {
        // Carreguem els tipus de factures
        NSMutableArray *mutableFetchResults  = [funcComunes setTipusFactures :managedObjectContext];
        [self setTipusFacturesArray:mutableFetchResults];
        // Refresh del tableview
        [_tblTipusFactures  reloadData];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}



- (void)viewDidLoad
{
    [super viewDidLoad];
    @try {
        // Intanciem comunes
        funcComunes = [ClComunes alloc];
        // Obtenim les dades necessaries per la gestió del Coredata
        managedObjectContext = [funcComunes get_mngObj:managedObjectContext];
        // Gestió de gestos per edició
        UITapGestureRecognizer *tapEdit = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handletapEdit:)];
        // Nombre de taps de editar
        tapEdit.numberOfTapsRequired = 2;
        // Afegim el gestor de "gestos" al tableview
        [_tblTipusFactures addGestureRecognizer:tapEdit];
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}

// Capturador gestos
-(IBAction)handletapEdit:(UITapGestureRecognizer *)cell
{
    @try {
        CGPoint tapLocation = [cell locationInView:self.tblTipusFactures];
        NSIndexPath *indexPath = [self.tblTipusFactures indexPathForRowAtPoint:tapLocation];
        if (indexPath) {
            // Cridem la pantalla de nova categoria pero en mode edicio
            ClScrNouTipusFactura *editTipusView = [self.storyboard instantiateViewControllerWithIdentifier:@"ViewNouTipusFactures"];
            // Activem el mode d'edicio
            editTipusView.edicio = @1;
            // Obtenim el id  a editar
            TipusFactura *lTipusFactura  = (TipusFactura *)[TipusFacturesArray  objectAtIndex: indexPath.row];
            editTipusView.TipusFacturaEdit  = lTipusFactura;
            editTipusView.navigationItem.prompt = @"Edició Tipus Factura";
            // Cridem la pantalla
            [self.navigationController pushViewController:editTipusView animated:YES];
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    // Return the number of rows in the section.
    return [TipusFacturesArray count];
    
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    
    
    static NSString *CellIdentifier = @"Cell";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier forIndexPath:indexPath];
    
    
    // Configure the cell...
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:@"Cell"];
    }
    // Obtenim el text del tipus d'iva
    TipusFactura *lTipusFactures = (TipusFactura *)[TipusFacturesArray objectAtIndex: indexPath.row];
    // Establim el text de la Cell
    [cell.textLabel setText:[lTipusFactures txt]];
    return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
  
    // Return NO if you do not want the specified item to be editable.
    return YES;
    
}


// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    NSError *error;
    @try {
        if (editingStyle == UITableViewCellEditingStyleDelete) {
            // Elimina les dades del Core Data
            TipusFactura *lTipusFactura  = (TipusFactura *)[TipusFacturesArray objectAtIndex: indexPath.row];
            // Abans d'eliminar, comprobem integritat referencial
            NSMutableArray *ref = [funcComunes get_references_tipusfactura:lTipusFactura :managedObjectContext];
            if ( [ref  count] > 0)
            {
                [funcComunes msg_pantalla:@"El Tipus de Factura encara s'utilitza, no es pot eliminar!"];
            }
            else
            {
                [managedObjectContext deleteObject:lTipusFactura];
                [managedObjectContext save:&error];
                // Elimina l'entrada de l'array
                [TipusFacturesArray removeObjectAtIndex:indexPath.row];
                // Elimina la fila del tableview
                [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
                [_tblTipusFactures reloadData];
            }
        }
        else if (editingStyle == UITableViewCellEditingStyleInsert) {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }
    }
    @catch (NSException *exception) {
        [funcComunes msg_excepcio:exception];
    }
}

@end
